import { world, ItemStack, GameMode } from '@minecraft/server';

// This array defines the tool items that will lose durability.
const toolTypeIds = [
    'old_villagers_plus:vindicator_axe',
    'old_villagers_plus:emerald_axe',
    'old_nether:ruby_axe',
    'old_nether:ruby_pickaxe',
    'old_nether:ruby_hoe',
    'old_nether:ruby_axe',
    'old_nether:ruby_shovel',
    'old_nether:taaffeite_axe',
    'old_nether:taaffeite_pickaxe',
    'old_nether:taaffeite_hoe',
    'old_nether:taaffeite_axe',
    'old_nether:taaffeite_shovel',
    'old_nether:taaffeite_mace',
    'old_nether:monochromatic_gold_axe',
    'old_nether:monochromatic_gold_pickaxe',
    'old_nether:monochromatic_gold_hoe',
    'old_nether:monochromatic_gold_axe',
    'old_nether:monochromatic_gold_shovel',
    'old_nether:monochromatic_gold_battle_axe',
    'old_nether:ruby_battle_axe'
];

// This array defines the weapon items that will lose durability.
const weaponTypeIds = [
    `old_villagers_plus:emerald_sword`,
    `old_nether:ruby_sword`,
    `old_nether:monochromatic_gold_sword`,
    `old_nether:taaffeite_sword`
];

// This array defines the vanilla items that will lose durability when mining an addon block.
const vanillaToolIds = [
    'minecraft:wooden_sword',
    'minecraft:wooden_shovel',
    'minecraft:wooden_pickaxe',
    'minecraft:wooden_axe',
    'minecraft:wooden_hoe',
    'minecraft:stone_sword',
    'minecraft:stone_shovel',
    'minecraft:stone_pickaxe',
    'minecraft:stone_axe',
    'minecraft:stone_hoe',
    'minecraft:iron_sword',
    'minecraft:iron_shovel',
    'minecraft:iron_pickaxe',
    'minecraft:iron_axe',
    'minecraft:iron_hoe',
    'minecraft:golden_sword',
    'minecraft:golden_shovel',
    'minecraft:golden_pickaxe',
    'minecraft:golden_axe',
    'minecraft:golden_hoe',
    'minecraft:diamond_sword',
    'minecraft:diamond_shovel',
    'minecraft:diamond_pickaxe',
    'minecraft:diamond_axe',
    'minecraft:diamond_hoe',
    'minecraft:netherite_sword',
    'minecraft:netherite_shovel',
    'minecraft:netherite_pickaxe',
    'minecraft:netherite_axe',
    'minecraft:netherite_hoe'
];

world.afterEvents.playerBreakBlock.subscribe(evd => {
    const { player, brokenBlockPermutation: block, itemStackBeforeBreak: itemUsed } = evd;

    // This returns if itemUsed is undefined or if the player is in Creative.
    if (!itemUsed || player.matches({ gameMode: GameMode.creative })) return;

    if (toolTypeIds.includes(itemUsed.typeId) || weaponTypeIds.includes(itemUsed.typeId)) {
        // This retrieves the player's equippable component.
        const playerEquippableComp = player.getComponent("equippable");

        // This returns if playerEquippableComp is undefined.
        if (!playerEquippableComp) return;

        // This retrieves the enchantable component of the item.
        const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
        const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;

        // Calculates the chance of an item breaking based on its unbreaking level. This is the vanilla unbreaking formula.
        const breakChance = 100 / (unbreakingLevel + 1);
        // Generates a random chance value between 0 and 100.
        const randomizeChance = Math.random() * 100;

        // This returns if breakChance is less than randomizeChance.
        if (breakChance < randomizeChance) return;

        // This retrieves the durability component of the item.
        const itemUsedDurabilityComp = itemUsed.getComponent("durability");

        // This returns if itemUsedDurabilityComp is undefined.
        if (!itemUsedDurabilityComp) return;

        let durabilityModifier = 0;
        if (toolTypeIds.includes(itemUsed.typeId)) {
            // If the item is a tool, then it will set the durability modifier to 1.
            durabilityModifier = 1;
        } else {
            // If the item is a weapon, then it will set the durability modifier to 2.
            durabilityModifier = 2;
        }

        // This will set the new durability value.
        itemUsedDurabilityComp.damage += durabilityModifier;

        // Declares and checks if the item is out of durability
        const maxDurability = itemUsedDurabilityComp.maxDurability
        const currentDamage = itemUsedDurabilityComp.damage
        if (currentDamage >= maxDurability) {

            // If the item is out of durability, plays the item breaking sound and removes the item
            player.playSound('random.break', { pitch: 1, location: player.location, volume: 1 })
            playerEquippableComp.setEquipment("Mainhand", new ItemStack('minecraft:air', 1));
        }
        else if (currentDamage < maxDurability) {

            // This sets the item in the player's selected slot.
            playerEquippableComp.setEquipment("Mainhand", itemUsed);
        }
    }
    else if (block.matches('old_villagers_plus:black_door') || block.matches('old_villagers_plus:yellow_door') || block.matches('old_villagers_plus:white_door') || block.matches('old_villagers_plus:red_door') || block.matches('old_villagers_plus:purple_door') || block.matches('old_villagers_plus:pink_door') || block.matches('old_villagers_plus:orange_door') || block.matches('old_villagers_plus:light_grey_door') || block.matches('old_villagers_plus:light_blue_door') || block.matches('old_villagers_plus:grey_door') || block.matches('old_villagers_plus:green_door') || block.matches('old_villagers_plus:dark_purple_door') || block.matches('old_villagers_plus:dark_green_door') || block.matches('old_villagers_plus:cyan_door') || block.matches('old_villagers_plus:brown_door') || block.matches('old_villagers_plus:blue_door') || block.matches('old_villagers_plus:black_trapdoor') || block.matches('old_villagers_plus:yellow_trapdoor') || block.matches('old_villagers_plus:white_trapdoor') || block.matches('old_villagers_plus:red_trapdoor') || block.matches('old_villagers_plus:purple_trapdoor') || block.matches('old_villagers_plus:pink_trapdoor') || block.matches('old_villagers_plus:orange_trapdoor') || block.matches('old_villagers_plus:light_grey_trapdoor') || block.matches('old_villagers_plus:light_blue_trapdoor') || block.matches('old_villagers_plus:grey_trapdoor') || block.matches('old_villagers_plus:green_trapdoor') || block.matches('old_villagers_plus:dark_purple_trapdoor') || block.matches('old_villagers_plus:dark_green_trapdoor') || block.matches('old_villagers_plus:cyan_trapdoor') || block.matches('old_villagers_plus:brown_trapdoor') || block.matches('old_villagers_plus:blue_trapdoor') || block.matches('old_villagers_plus:black_fence') || block.matches('old_villagers_plus:yellow_fence') || block.matches('old_villagers_plus:white_fence') || block.matches('old_villagers_plus:red_fence') || block.matches('old_villagers_plus:purple_fence') || block.matches('old_villagers_plus:pink_fence') || block.matches('old_villagers_plus:orange_fence') || block.matches('old_villagers_plus:light_grey_fence') || block.matches('old_villagers_plus:light_blue_fence') || block.matches('old_villagers_plus:grey_fence') || block.matches('old_villagers_plus:green_fence') || block.matches('old_villagers_plus:dark_purple_fence') || block.matches('old_villagers_plus:dark_green_fence') || block.matches('old_villagers_plus:cyan_fence') || block.matches('old_villagers_plus:brown_fence') || block.matches('old_villagers_plus:blue_fence') || block.matches('old_villagers_plus:black_fence_gate') || block.matches('old_villagers_plus:yellow_fence_gate') || block.matches('old_villagers_plus:white_fence_gate') || block.matches('old_villagers_plus:red_fence_gate') || block.matches('old_villagers_plus:purple_fence_gate') || block.matches('old_villagers_plus:pink_fence_gate') || block.matches('old_villagers_plus:orange_fence_gate') || block.matches('old_villagers_plus:light_grey_fence_gate') || block.matches('old_villagers_plus:light_blue_fence_gate') || block.matches('old_villagers_plus:grey_fence_gate') || block.matches('old_villagers_plus:green_fence_gate') || block.matches('old_villagers_plus:dark_purple_fence_gate') || block.matches('old_villagers_plus:dark_green_fence_gate') || block.matches('old_villagers_plus:cyan_fence_gate') || block.matches('old_villagers_plus:brown_fence_gate') || block.matches('old_villagers_plus:blue_fence_gate') || block.matches('old_villagers_plus:black_planks') || block.matches('old_villagers_plus:yellow_planks') || block.matches('old_villagers_plus:white_planks') || block.matches('old_villagers_plus:red_planks') || block.matches('old_villagers_plus:purple_planks') || block.matches('old_villagers_plus:pink_planks') || block.matches('old_villagers_plus:orange_planks') || block.matches('old_villagers_plus:light_grey_planks') || block.matches('old_villagers_plus:light_blue_planks') || block.matches('old_villagers_plus:grey_planks') || block.matches('old_villagers_plus:green_planks') || block.matches('old_villagers_plus:dark_purple_planks') || block.matches('old_villagers_plus:dark_green_planks') || block.matches('old_villagers_plus:cyan_planks') || block.matches('old_villagers_plus:brown_planks') || block.matches('old_villagers_plus:blue_planks') || block.matches('old_villagers_plus:black_slab') || block.matches('old_villagers_plus:yellow_slab') || block.matches('old_villagers_plus:white_slab') || block.matches('old_villagers_plus:red_slab') || block.matches('old_villagers_plus:purple_slab') || block.matches('old_villagers_plus:pink_slab') || block.matches('old_villagers_plus:orange_slab') || block.matches('old_villagers_plus:light_grey_slab') || block.matches('old_villagers_plus:light_blue_slab') || block.matches('old_villagers_plus:grey_slab') || block.matches('old_villagers_plus:green_slab') || block.matches('old_villagers_plus:dark_purple_slab') || block.matches('old_villagers_plus:dark_green_slab') || block.matches('old_villagers_plus:cyan_slab') || block.matches('old_villagers_plus:brown_slab') || block.matches('old_villagers_plus:blue_slab') || block.matches('old_villagers_plus:love_block')) {
        if (vanillaToolIds.includes(itemUsed.typeId)) {

            // This retrieves the player's equippable component.
            const playerEquippableComp = player.getComponent("equippable");

            // This returns if playerEquippableComp is undefined.
            if (!playerEquippableComp) return;

            // This retrieves the enchantable component of the item.
            const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
            const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;

            // Calculates the chance of an item breaking based on its unbreaking level. This is the vanilla unbreaking formula.
            const breakChance = 100 / (unbreakingLevel + 1);
            // Generates a random chance value between 0 and 100.
            const randomizeChance = Math.random() * 100;

            // This returns if breakChance is less than randomizeChance.
            if (breakChance < randomizeChance) return;

            // This retrieves the durability component of the item.
            const itemUsedDurabilityComp = itemUsed.getComponent("durability");

            // This returns if itemUsedDurabilityComp is undefined.
            if (!itemUsedDurabilityComp) return;

            // This will set the new durability value.
            itemUsedDurabilityComp.damage += 1;

            // Declares and checks if the item is out of durability
            const maxDurability = itemUsedDurabilityComp.maxDurability
            const currentDamage = itemUsedDurabilityComp.damage
            if (currentDamage >= maxDurability) {

                // If the item is out of durability, plays the item breaking sound and removes the item
                player.playSound('random.break', { pitch: 1, location: player.location, volume: 1 })
                playerEquippableComp.setEquipment("Mainhand", new ItemStack('minecraft:air', 1));
            }
            else if (currentDamage < maxDurability) {

                // This sets the item in the player's selected slot.
                playerEquippableComp.setEquipment("Mainhand", itemUsed);
            }
        }
    }
})